@@ -7,10 +7,10 @@ use crate::parser::types::Field;
 use crate::resolver_utils::resolve_list;
 use crate::{
     registry, ContextSelectionSet, InputType, InputValueError, InputValueResult, OutputType,
-    Positioned, Result, ServerResult, Type, Value,
+    Positioned, Result, ServerResult, Value,
 };
 
-impl<T: Type> Type for HashSet<T> {
+impl<T: InputType + Hash + Eq> InputType for HashSet<T> {
     fn type_name() -> Cow<'static, str> {
         Cow::Owned(format!("[{}]", T::qualified_type_name()))
     }
@@ -23,9 +23,7 @@ impl<T: Type> Type for HashSet<T> {
         T::create_type_info(registry);
         Self::qualified_type_name()
     }
-}
 
-impl<T: InputType + Hash + Eq> InputType for HashSet<T> {
     fn parse(value: Option<Value>) -> InputValueResult<Self> {
         match value.unwrap_or_default() {
             Value::List(values) => values
@@ -48,6 +46,19 @@ impl<T: InputType + Hash + Eq> InputType for HashSet<T> {
 
 #[async_trait::async_trait]
 impl<T: OutputType + Hash + Eq> OutputType for HashSet<T> {
+    fn type_name() -> Cow<'static, str> {
+        Cow::Owned(format!("[{}]", T::qualified_type_name()))
+    }
+
+    fn qualified_type_name() -> String {
+        format!("[{}]!", T::qualified_type_name())
+    }
+
+    fn create_type_info(registry: &mut registry::Registry) -> String {
+        T::create_type_info(registry);
+        Self::qualified_type_name()
+    }
+
     async fn resolve(
         &self,
         ctx: &ContextSelectionSet<'_>,
